// ==================================================================
// @(#)main-test.c
//
// Main source file for cbgp-test application.
//
// @author Bruno Quoitin (bqu@info.ucl.ac.be)
// @lastdate 21/05/2007
// @date 21/05/07
// ==================================================================

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <stdlib.h>

#include <libgds/log.h>

#include <api.h>
#include <bgp/as.h>
#include <bgp/comm.h>
#include <bgp/comm_hash.h>
#include <bgp/route.h>
#include <net/prefix.h>

// -----[ main_test ]------------------------------------------------
/**
 * Test function (only available in EXPERIMENTAL mode).
 *
 * Current test function measures the number of nodes required to
 * store a complete RIB into a non-compact trie (radix-tree) or in a
 * compact trie (Patricia tree).
 */
#ifdef __EXPERIMENTAL__
int main_test(char * pcArg)
{
  int iIndex;
  SRoutes * pRoutes= mrtd_ascii_load_routes(NULL, pcArg);
  SRoute * pRoute;
  SRadixTree * pRadixTree;
  STrie * pTrie;

  if (pRoutes != NULL) {
    pRadixTree= radix_tree_create(32, NULL);
    pTrie= trie_create(NULL);
    for (iIndex= 0; iIndex < routes_list_get_num(pRoutes); iIndex++) {
      pRoute= (SRoute *) pRoutes->data[iIndex];
      radix_tree_add(pRadixTree, pRoute->sPrefix.tNetwork,
		     pRoute->sPrefix.uMaskLen, pRoute);
      trie_insert(pTrie, pRoute->sPrefix.tNetwork,
		  pRoute->sPrefix.uMaskLen, pRoute);
    }
    fprintf(stdout, "radix-tree: %d\n", radix_tree_num_nodes(pRadixTree));
    fprintf(stdout, "trie: %d\n", trie_num_nodes(pTrie));
    radix_tree_destroy(&pRadixTree);
    trie_destroy(&pTrie);
    routes_list_destroy(&pRoutes);
    return 0;
  } else {
    fprintf(stderr, "could not load \"%s\"\n", pcArg);
    return -1;
  }
}
#endif

// -----[ _main_init ]-----------------------------------------------
static void _main_init() __attribute((constructor));
static void _main_init()
{
  libcbgp_init();
}

// -----[ _main_done ]-----------------------------------------------
static void _main_done() __attribute((destructor));
static void _main_done()
{
  libcbgp_done();
}

// -----[ main ]-----------------------------------------------------
int main(int argc, char * argv[])
{
  SRoute * pRoute1, * pRoute2;
  SCommunities * pComm1, * pComm2;

  SPrefix sPrefix= { .tNetwork= IPV4_TO_INT(130,104,0,0),
		     .uMaskLen= 16 };

  pRoute1= route_create(sPrefix, NULL, IPV4_TO_INT(1,0,0,0), ROUTE_ORIGIN_IGP);
  pRoute2= route_create(sPrefix, NULL, IPV4_TO_INT(2,0,0,0), ROUTE_ORIGIN_IGP);

  pComm1= comm_create();
  pComm2= comm_copy(pComm1);

  log_printf(pLogOut, "%p =!= %p\n", pComm1, pComm2);

  route_set_comm(pRoute1, pComm1);
  route_set_comm(pRoute2, pComm2);

  BGP_OPTIONS_SHOW_MODE= BGP_ROUTES_OUTPUT_MRT_ASCII;
  route_dump(pLogOut, pRoute1); log_printf(pLogOut, "\n");
  route_dump(pLogOut, pRoute2); log_printf(pLogOut, "\n");

  log_printf(pLogOut, "%p =?= %p\n",
	     pRoute1->pAttr->pCommunities, pRoute2->pAttr->pCommunities);

  comm_hash_content(pLogOut);

  route_destroy(&pRoute1);

  comm_hash_content(pLogOut);

  route_destroy(&pRoute2);

  return EXIT_SUCCESS;
}
